package jp.co.ecosysnetwork.ccloca_app;

import android.app.DatePickerDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.text.InputType;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.View;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.DialogFragment;
import androidx.fragment.app.FragmentActivity;

import java.util.Calendar;

/**
 * ダイアログのホストにイベントを渡すダイアログ
 */
public class NoticeDialogFragment extends DialogFragment {
    //region インタフェース
    /**
     * イベントコールバックリスナ
     */
    public interface NoticeDialogListener {
        /**
         * OKボタン押下コールバック
         * @param dialog ダイアログ
         */
        public void onDialogPositiveClick(NoticeDialogFragment dialog);
        /**
         * キャンセルボタン押下コールバック
         * @param dialog ダイアログ
         */
        public void onDialogNegativeClick(NoticeDialogFragment dialog);
        /**
         * ニュートラルボタン押下コールバック
         * @param dialog ダイアログ
         */
        public void onDialogNeutralClick(NoticeDialogFragment dialog);
    }
    //endregion
    //region メンバ変数
    /**
     * 自身のアクティビティ
     */
    private FragmentActivity _activity;
    /**
     * 自身のコンテキスト
     */
    private Context _context;
    /**
     * イベント通知リスナ
     */
    private NoticeDialogListener _listener;
    /**
     * ダイアログタイトル
     */
    private String _title;
    /**
     * ダイアログメッセージ
     */
    private String _message;
    /**
     * 編集用テキストを表示するか否か
     */
    private boolean _isShowEdit;
    /**
     * 編集用テキストに入力された文字を隠すか否か
     */
    private boolean _isEditPassword;
    /**
     * OKボタンテキスト
     */
    private String _posBtnText;
    /**
     * キャンセルボタンテキスト
     */
    private String _negBtnText;
    /**
     * ニュートラルボタンテキスト
     */
    private String _ntrBtnText;
    /**
     * 入力エディットテキスト
     */
    private EditText _edtInput;
    /**
     * 日付選択テキストを表示するか否か
     */
    private boolean _isShowDatePicker;
    /**
     * 日付選択エディットテキスト
     */
    private EditText _edtDate;
    //endregion
    //region 生成と廃棄
    /**
     * コンストラクタ
     * @param title ダイアログタイトル
     * @param message ダイアログメッセージ
     * @param isShowEdit 編集用テキストを表示するか否か
     * @param isEditPassword 編集用テキストに入力された文字を隠すか否か
     * @param posBtnText OKボタンテキスト
     * @param negBtnText キャンセルボタンテキスト
     */
    public NoticeDialogFragment(String title, String message, boolean isShowEdit, boolean isEditPassword, String posBtnText, String negBtnText) {
        _title = title;
        _message = message;
        _isShowEdit = isShowEdit;
        _isEditPassword = isEditPassword;
        _posBtnText = posBtnText;
        _negBtnText = negBtnText;
        _ntrBtnText = "";
        _edtInput = null;
        _isShowDatePicker = false;
    }
    /**
     * コンストラクタ
     * @param title ダイアログタイトル
     * @param message ダイアログメッセージ
     * @param isShowEdit 編集用テキストを表示するか否か
     * @param isEditPassword 編集用テキストに入力された文字を隠すか否か
     * @param posBtnText OKボタンテキスト
     * @param negBtnText キャンセルボタンテキスト
     * @param ntrBtnText ニュートラルボタンテキスト
     */
    public NoticeDialogFragment(String title, String message, boolean isShowEdit, boolean isEditPassword, String posBtnText, String negBtnText, String ntrBtnText) {
        _title = title;
        _message = message;
        _isShowEdit = isShowEdit;
        _isEditPassword = isEditPassword;
        _posBtnText = posBtnText;
        _negBtnText = negBtnText;
        _ntrBtnText = ntrBtnText;
        _edtInput = null;
        _isShowDatePicker = false;
    }
    /**
     * コンストラクタ
     * @param title ダイアログタイトル
     * @param message ダイアログメッセージ
     * @param isShowEdit 編集用テキストを表示するか否か
     * @param isEditPassword 編集用テキストに入力された文字を隠すか否か
     * @param posBtnText OKボタンテキスト
     * @param negBtnText キャンセルボタンテキスト
     * @param ntrBtnText ニュートラルボタンテキスト
     * @param isShowDatePicker 日付選択を可能にするか
     */
    public NoticeDialogFragment(String title, String message, boolean isShowEdit, boolean isEditPassword, String posBtnText, String negBtnText, String ntrBtnText, boolean isShowDatePicker) {
        _title = title;
        _message = message;
        _isShowEdit = isShowEdit;
        _isEditPassword = isEditPassword;
        _posBtnText = posBtnText;
        _negBtnText = negBtnText;
        _ntrBtnText = ntrBtnText;
        _edtInput = null;
        _isShowDatePicker = isShowDatePicker;
    }
    //endregion
    //region プロパティ
    /**
     * 入力エディットテキストのgetter
     * @return 入力エディットテキスト
     */
    public EditText get_edtInput() {
        return _edtInput;
    }
    /**
     * 日付選択エディットテキストのgetter
     * @return 日付選択エディットテキスト
     */
    public EditText get_edtDate() {return _edtDate;}
    //endregion
    //region イベント
    /**
     * アタッチイベント
     * @param context ホストのコンテキスト
     */
    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        // コールバックリスナが登録されているかのチェック
        try {
            _activity = getActivity();
            _context = getContext();
            _listener = (NoticeDialogListener) context;
        } catch (ClassCastException e) {
            throw new ClassCastException(context.toString() + " must implement NoticeDialogListener");
        }
    }
    /**
     * ダイアログ作成イベント
     * @param savedInstanceState アクティビティが以前にシャットダウンされた後に再初期化されている場合、このバンドルにはonSaveInstanceState（Bundle）で最後に提供されたデータが含まれます。 注：それ以外の場合はnullです。
     * @return ダイアログ
     */
    @Override
    public @NonNull Dialog onCreateDialog(Bundle savedInstanceState) {
        AlertDialog.Builder builder = new AlertDialog.Builder(_activity);

        // ダイアログタイトル
        if (!_title.equals("")) {
            TextView titleView = new TextView(this.getContext());
            titleView.setText(_title);
            titleView.setTextSize(TypedValue.COMPLEX_UNIT_PX, getResources().getDimension(R.dimen.dialogTitleTextSize));
            titleView.setTextColor(ContextCompat.getColor(_context, R.color.dialogTitleTextColor));
            titleView.setBackgroundColor(ContextCompat.getColor(_context, R.color.dialogTitleColor));
            int pad = (int)getResources().getDimension(R.dimen.dialogTitlePadding);
            titleView.setPadding(pad, pad, pad, pad);
            titleView.setGravity(Gravity.CENTER);
            builder.setCustomTitle(titleView);
        }

        // ボディ部の垂直リニアレイアウト
        LinearLayout lnlBody = null;
        if (!_message.equals("") || _isShowEdit) {
            lnlBody = new LinearLayout(this.getContext());
            LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams(
                    LinearLayout.LayoutParams.WRAP_CONTENT,
                    LinearLayout.LayoutParams.WRAP_CONTENT);
            lnlBody.setLayoutParams(lp);
            lnlBody.setOrientation(LinearLayout.VERTICAL);
            int pad = (int)getResources().getDimension(R.dimen.dialogMsgPadding);
            lnlBody.setPadding(pad, pad, pad, pad);
        }

        // ダイアログメッセージ
        if (!_message.equals("")) {
            TextView msgView = new TextView(this.getContext());
            msgView.setText(_message);
            msgView.setTextSize(TypedValue.COMPLEX_UNIT_PX, getResources().getDimension(R.dimen.dialogMsgTextSize));
            msgView.setTextColor(ContextCompat.getColor(_context, R.color.dialogMsgTextColor));
            msgView.setBackgroundColor(ContextCompat.getColor(_context, R.color.dialogMsgColor));
            int pad = (int)getResources().getDimension(R.dimen.dialogMsgPadding);
            msgView.setPadding(0, 0, 0, pad);
            lnlBody.addView(msgView);
        }

        // 編集用テキスト
        if (_isShowEdit) {
            _edtInput = new EditText(this.getContext());
            _edtInput.setTextSize(TypedValue.COMPLEX_UNIT_PX, getResources().getDimension(R.dimen.dialogMsgTextSize));
            _edtInput.setTextColor(ContextCompat.getColor(_context, R.color.editTextColor));
            _edtInput.setBackground(_activity.getDrawable(R.drawable.bg_edittext));
            int pad = (int)getResources().getDimension(R.dimen.dialogEditPadding);
            _edtInput.setPadding(pad, pad, pad, pad);
            if (_isEditPassword) {
                _edtInput.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_PASSWORD);
            }
            lnlBody.addView(_edtInput);
        }

        // 日付選択
        if (_isShowDatePicker) {
            _edtDate = new EditText(this.getContext());
            _edtDate.setTextSize(TypedValue.COMPLEX_UNIT_PX, getResources().getDimension(R.dimen.dialogMsgTextSize));
//            _edtDate.setTextColor(ContextCompat.getColor(_context, R.color.editTextColor));
//            _edtDate.setBackground(_activity.getDrawable(R.drawable.bg_edittext));
            _edtDate.setFocusable(false);
            int pad = (int)getResources().getDimension(R.dimen.dialogEditPadding);
            _edtDate.setPadding(pad, pad, pad, pad);
            _edtDate.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    //Calendarインスタンスを取得
                    final Calendar date = Calendar.getInstance();
                    //DatePickerDialogインスタンスを取得
                    DatePickerDialog datePickerDialog = new DatePickerDialog(
                            _context,
                            new DatePickerDialog.OnDateSetListener() {
                                @Override
                                public void onDateSet(DatePicker view, int year, int month, int dayOfMonth) {
                                    //setした日付を取得して表示
                                    _edtDate.setText(String.format("%d/%02d/%02d", year, month+1, dayOfMonth));
                                }
                            },
                            date.get(Calendar.YEAR),
                            date.get(Calendar.MONTH),
                            date.get(Calendar.DATE)
                    );
                    //dialogを表示
                    datePickerDialog.show();
                }
            });
            lnlBody.addView(_edtDate);
        }

        // ボディ部の垂直リニアレイアウトを設定
        if (lnlBody != null) {
            builder.setView(lnlBody);
        }

        // OKボタン
        if (!_posBtnText.equals("")) {
            builder.setPositiveButton(_posBtnText, new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int id) {
                    _listener.onDialogPositiveClick(NoticeDialogFragment.this);
                }
            });
        }

        // キャンセルボタン
        if (!_negBtnText.equals("")) {
            builder.setNegativeButton(_negBtnText, new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int id) {
                    _listener.onDialogNegativeClick(NoticeDialogFragment.this);
                }
            });
        }

        // ニュートラルボタン
        if (!_ntrBtnText.equals("")) {
            builder.setNeutralButton(_ntrBtnText, new DialogInterface.OnClickListener() {
                public void onClick(DialogInterface dialog, int id) {
                    _listener.onDialogNeutralClick(NoticeDialogFragment.this);
                }
            });
        }

        return builder.create();
    }
    /**
     * アクティビティ開始イベント
     */
    @Override
    public void onStart() {
        super.onStart();

        AlertDialog ald = (AlertDialog)this.getDialog();
        int margin = (int)getResources().getDimension(R.dimen.dialogBtnMargin);

        // OKボタン
        Button btnPos = ald.getButton(AlertDialog.BUTTON_POSITIVE);
        btnPos.setTextSize(TypedValue.COMPLEX_UNIT_PX, getResources().getDimension(R.dimen.dialogBtnTextSize));
        btnPos.setTextColor(ContextCompat.getColor(_context, R.color.dialogBtnTextColor));
        btnPos.setBackground(_activity.getDrawable(R.drawable.bg_button));
        btnPos.setWidth((int)getResources().getDimensionPixelSize(R.dimen.dialogBtnWidth));
        btnPos.setHeight((int)getResources().getDimensionPixelSize(R.dimen.dialogBtnHeight));

        LinearLayout.LayoutParams lpPos = (LinearLayout.LayoutParams)btnPos.getLayoutParams();
        lpPos.setMargins(0, 0, margin, 0);
        btnPos.setLayoutParams(lpPos);

        // キャンセルボタン
        Button btnNeg = ald.getButton(AlertDialog.BUTTON_NEGATIVE);
        btnNeg.setTextSize(TypedValue.COMPLEX_UNIT_PX, getResources().getDimension(R.dimen.dialogBtnTextSize));
        btnNeg.setTextColor(ContextCompat.getColor(_context, R.color.dialogBtnTextColor));
        btnNeg.setBackground(_activity.getDrawable(R.drawable.bg_button));
        btnNeg.setWidth((int)getResources().getDimensionPixelSize(R.dimen.dialogBtnWidth));
        btnNeg.setHeight((int)getResources().getDimensionPixelSize(R.dimen.dialogBtnHeight));

        LinearLayout.LayoutParams lpNeg = (LinearLayout.LayoutParams)btnNeg.getLayoutParams();
        lpNeg.setMargins(0, 0, margin, 0);
        btnNeg.setLayoutParams(lpNeg);

        // ニュートラルボタン
        Button btnNtr = ald.getButton(AlertDialog.BUTTON_NEUTRAL);
        btnNtr.setTextSize(TypedValue.COMPLEX_UNIT_PX, getResources().getDimension(R.dimen.dialogBtnTextSize));
        btnNtr.setTextColor(ContextCompat.getColor(_context, R.color.dialogBtnTextColor));
        btnNtr.setBackground(_activity.getDrawable(R.drawable.bg_button));
        btnNtr.setWidth((int)getResources().getDimensionPixelSize(R.dimen.dialogBtnWidth));
        btnNtr.setHeight((int)getResources().getDimensionPixelSize(R.dimen.dialogBtnHeight));

        LinearLayout.LayoutParams lpNtr = (LinearLayout.LayoutParams)btnNtr.getLayoutParams();
        lpNtr.setMargins(margin, 0, 0, 0);
        btnNtr.setLayoutParams(lpNtr);

        // ボタンのマージンを設定するとダイアログサイズが変わらずはみ出すため、親ビューにパディングを設定
        View viwParent = (View)btnPos.getParent();
        viwParent.setPadding(0, margin, 0, margin);
    }
    /**
     * 携帯端末ーがアクティビティを離れることを最初に示すイベント
     */
    @Override
    public void onPause() {
        super.onPause();
        //dismiss();
    }
    //endregion
}
