package jp.co.ecosysnetwork.ccloca_app.activity;

import androidx.fragment.app.DialogFragment;

import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.util.Xml;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.Spinner;
import android.widget.TextView;

import org.xmlpull.v1.XmlPullParser;

import java.io.DataOutputStream;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import jp.co.ecosysnetwork.ccloca_app.MyApplication;
import jp.co.ecosysnetwork.ccloca_app.NoticeDialogFragment;
import jp.co.ecosysnetwork.ccloca_app.R;
import jp.co.ecosysnetwork.ccloca_app.adapter.BusRouteAdapter;
import jp.co.ecosysnetwork.ccloca_app.models.busRoute_info;
import jp.co.ecosysnetwork.ccloca_app.models.busRoute_infos;
import jp.co.ecosysnetwork.ccloca_app.models.course_settings;
import jp.co.ecosysnetwork.ccloca_app.models.drive_state;
import jp.co.ecosysnetwork.ccloca_app.models.system_settings;
import jp.co.ecosysnetwork.universal.EsnDebugLog;
import jp.co.ecosysnetwork.universal.EsnUtility;
import jp.co.ecosysnetwork.universal.EsnXmlParser;

/**
 * コース設定アクティビティ
 */
public class CourseSettingsActivity
        extends BaseActivity
        implements View.OnClickListener,
        NoticeDialogFragment.NoticeDialogListener{
    //region 定数
    /**
     * 保存確認ダイアログ
     */
    private static final String DIALOG_TAG_SAVE = "cfmSave";
    /**
     * コース更新確認ダイアログ
     */
    private static final String DIALOG_TAG_UPDATE_COURSE = "cfmUpdateCourse";
    /**
     * コース更新成功ダイアログ
     */
    private static final String DIALOG_TAG_UPDATE_COURSE_SUCCESS = "UpdateCourseSuccess";
    /**
     * コース更新成功の保存失敗ダイアログ
     */
    private static final String DIALOG_TAG_UPDATE_COURSE_SUCCESS_SAVE_ERROR = "UpdateCourseSuccessSaveError";
    /**
     * コース更新失敗ダイアログ
     */
    private static final String DIALOG_TAG_UPDATE_COURSE_ERROR = "UpdateCourseError";
    /**
     * コース更新要求ダイアログ
     */
    private static final String DIALOG_TAG_RESET_COURSE = "ResetCourse";
    /**
     * コース要求後の待機時間
     */
    private static final int WAIT_MSEC_REQ_COURSE_SEND = 1000;
    //endregion
    //region メンバ変数
    /**
     * シングルトンのアプリケーション参照
     */
    private final MyApplication _app = MyApplication.getInstance();
    /**
     * コース設定リストビュー
     */
    private Spinner _spnCourseSettings;
    /**
     * 保存ボタン
     */
    private Button _btnSave;
    /**
     * コース更新ボタン
     */
    private Button _btnUdpateCourse;
    /**
     * コース更新プログレスバー
     */
    private ProgressBar _prgUpdateCourse;
    /**
     * 保存元の値
     */
    private course_settings _org_model;
    /**
     * 編集中の値
     */
    private course_settings _tmp_model;
    /**
     * コース情報リストモデル
     */
    private busRoute_infos _busRoute_infos;
    /**
     * システム設定モデル
     */
    private system_settings _system_settings;
    /**
     * 通信中フラグ
     */
    private boolean _communicating;
    //endregion
    //region 内部メソッド
    /**
     * リフレッシュ（ボタン）
     */
    private void refreshButton() {
        // 保存ボタン
        _btnSave.setEnabled(!_communicating &&
                !_org_model.equals(_tmp_model));

        // コース更新ボタン
        boolean enaFlg = !_communicating &&
                (_system_settings.phone_id != null) &&
                (_system_settings.phone_id.length() > 0);
        _btnUdpateCourse.setEnabled(enaFlg);
    }
    /**
     * 終了処理
     */
    private void terminate() {
    }
    /**
     * 戻るボタン押下
     */
    private void clickBtnBack() {
        EsnDebugLog.outputLog("戻るボタン押下", EsnDebugLog.DIV.INFO);

        terminate();
        finishAndRemoveTask();
    }
    /**
     * 保存確認ダイアログ：はいボタン押下
     */
    private void clickPosCfmSave() {
        // システム設定の保存処理
        if (_tmp_model.savePreference()) {
            EsnDebugLog.outputLog(String.format(Locale.JAPAN,"コース設定の保存処理成功 %s", _tmp_model.toLogString()), EsnDebugLog.DIV.WARN);

            // リフレッシュ
            _org_model = _tmp_model.clone();
            refreshButton();
        }
        else {
            EsnDebugLog.outputLog(String.format(Locale.JAPAN,"コース設定の保存処理失敗 %s", _tmp_model.toLogString()), EsnDebugLog.DIV.ERR);
        }
    }
    /**
     *  保存ボタン押下
     */
    private void clickBtnSave() {
        EsnDebugLog.outputLog("保存ボタン押下", EsnDebugLog.DIV.INFO);

        // 変更データがない場合は処理しない
        if (_org_model.equals(_tmp_model)) {
            EsnDebugLog.outputLog("変更データが存在しないため保存しない", EsnDebugLog.DIV.WARN);
            return;
        }

        // 保存確認ダイアログ表示
        DialogFragment newFragment = new NoticeDialogFragment(getString(R.string.titleCfm),
                getString(R.string.msgCourseSettingSave),
                false,
                false,
                getString(R.string.yes),
                getString(R.string.no)
        );
        newFragment.show(getSupportFragmentManager(), DIALOG_TAG_SAVE);
    }
    /**
     * コース更新確認ダイアログ：はいボタン押下
     */
    private void clickPosCfmUpdateCourse() {
        // プログレスバー表示
        _communicating = true;
        refreshButton();
        _prgUpdateCourse.setVisibility(View.VISIBLE);

        final Handler handler = new Handler(Looper.getMainLooper());
        ExecutorService executor = Executors.newSingleThreadExecutor();
        executor.execute(new Runnable() {
            /**
             * 携帯端末認証バックグラウンド処理
             */
            @Override
            public void run() {
                final String ret = updateCourseExec();
                handler.postDelayed(new Runnable() {
                    /**
                     * バックグラウンド終了後のUI処理
                     */
                    @Override
                    public void run() {
                        // プログレスバー消す
                        _communicating = false;
                        _prgUpdateCourse.setVisibility(View.GONE);
                        if (ret == null) {
                            // コース更新成功
                            if (_busRoute_infos.savePreference()) {
                                EsnDebugLog.outputLog(String.format(Locale.JAPAN,"コース更新成功 %s", _busRoute_infos.toLogString()), EsnDebugLog.DIV.WARN);

                                // 成功音声を再生
                                _app.playWav(R.raw.tochaku, 500);

                                // コース情報更新
                                BusRouteAdapter adp2 = new BusRouteAdapter(_app.getApplicationContext(), (List<busRoute_info>)_busRoute_infos.items);
                                _spnCourseSettings.setAdapter(adp2);

                                SimpleDateFormat fmtYYYYMMDD = new SimpleDateFormat("yyyyMMdd", Locale.JAPANESE);
                                String nowDate = fmtYYYYMMDD.format(new Date());
                                EsnDebugLog.outputLog(String.format(Locale.JAPAN, "日替わり処理 %s -> %s", _app.drive_state_model.getLastUpdateCourseDate(), nowDate), EsnDebugLog.DIV.WARN);
                                _app.drive_state_model.setLastUpdateCourseDate(nowDate, true);
                                // コース更新成功ダイアログ
                                DialogFragment newFragment = new NoticeDialogFragment(getString(R.string.titleCfm),
                                        getString(R.string.msgSuccessUpdateCourse),
                                        false,
                                        false,
                                        getString(R.string.ok),
                                        "");
                                newFragment.show(getSupportFragmentManager(), DIALOG_TAG_UPDATE_COURSE_SUCCESS);
                            }
                            else {
                                EsnDebugLog.outputLog(String.format(Locale.JAPAN,"コース情報の保存処理失敗 %s", _busRoute_infos.toLogString()), EsnDebugLog.DIV.ERR);

                                // 認証成功後の携帯端末ID保存失敗ダイアログ
                                DialogFragment newFragment = new NoticeDialogFragment(getString(R.string.titleError),
                                        getString(R.string.msgSuccessUpdateCourseButSaveError),
                                        false,
                                        false,
                                        getString(R.string.ok),
                                        "");
                                newFragment.show(getSupportFragmentManager(), DIALOG_TAG_UPDATE_COURSE_SUCCESS_SAVE_ERROR);
                            }
                        }
                        else {
                            // 認証失敗ダイアログ
                            DialogFragment newFragment = new NoticeDialogFragment(getString(R.string.titleError),
                                    ret,
                                    false,
                                    false,
                                    getString(R.string.ok),
                                    "");
                            newFragment.show(getSupportFragmentManager(), DIALOG_TAG_UPDATE_COURSE_ERROR);
                        }
                        // リフレッシュ
                        refreshButton();
                    }
                }, 500);
            }
        });
    }
    /**
     *  コース更新ボタン押下
     */
    private void clickBtnUpdateCourse() {
        EsnDebugLog.outputLog("コース更新ボタン押下", EsnDebugLog.DIV.INFO);

        // コース更新確認ダイアログ表示
        DialogFragment newFragment = new NoticeDialogFragment(getString(R.string.titleCfm),
                getString(R.string.msgUpdateCourse),
                false,
                false,
                getString(R.string.yes),
                getString(R.string.no)
        );
        newFragment.show(getSupportFragmentManager(), DIALOG_TAG_UPDATE_COURSE);
    }
    //region コース更新処理
    /**
     * 携帯端末認証実行処理
     * @return エラーメッセージ
     */
    private String updateCourseExec(){
        String ret = null;

        HttpURLConnection con = null;
        DataOutputStream os = null;
        InputStream is = null;
        try {
            URL accUrl = new URL(_system_settings.server_url + MyApplication.APIURL_GET_COURSE);
            con = (HttpURLConnection)accUrl.openConnection();
            con.setConnectTimeout(10 * 1000);
            con.setRequestMethod("POST");
            con.setDoOutput(true);
            con.setDoInput(true);
            con.setUseCaches(false);

            // ヘッダ設定
            con.setRequestProperty("ANDROID-API-KEY", MyApplication.API_KEY);
            con.setRequestProperty("Connection", "Keep-Alive");
            con.setRequestProperty("Charset", "UTF-8");
            String boundary = "-----------------------------" + System.currentTimeMillis();
            con.setRequestProperty("Content-Type", "multipart/form-data; boundary=" + boundary);
            con.connect();

            // 送信データの設定
            os = new DataOutputStream(con.getOutputStream());
            append_string(os, boundary, "intFacilityPhoneNo", _system_settings.phone_id);
            append_final(os, boundary);
            os = null;
            // 携帯端末認証送信後のウェイト
            MyApplication.sleep(WAIT_MSEC_REQ_COURSE_SEND);

            // 受信データを取得
            is = con.getInputStream();
            // 受信データのXMLパース
            XmlPullParser parser = Xml.newPullParser();
            parser.setInput(is, "UTF-8");

            // 受信データのパース
            EsnXmlParser esnParser = new EsnXmlParser(parser);
            ret = esnParser.tryParse();
            if (ret == null) {
                // パース成功
                busRoute_info tmpBusRouteInfo = null;
                List<busRoute_info> tmpBusRouteInfos = new ArrayList<>();
                busRoute_info info = new busRoute_info();
                info.bus_route_no = "0";
                info.bus_route_name = "コースを選択してください";
                tmpBusRouteInfos.add(info);
                for (EsnXmlParser.ParseItem item : esnParser.getParseItems()) {
                    if (item.tag.equals("intBusRouteNo")) {
                        // コースのID
                        if (!EsnUtility.isNumeric(item.value)){
                            ret = String.format(Locale.JAPAN, "BusRouteNoが数値でない。 %s=%s", item.tag, item.value);
                            EsnDebugLog.outputLog(ret, EsnDebugLog.DIV.ERR);
                            break;
                        }
                        if (Integer.parseInt(item.value) < 0) {
                            ret = String.format(Locale.JAPAN, "BusRouteNoの値が正常でない。 %s=%s", item.tag, item.value);
                            EsnDebugLog.outputLog(ret, EsnDebugLog.DIV.ERR);
                            break;
                        }
                        tmpBusRouteInfo = new busRoute_info();
                        tmpBusRouteInfo.bus_route_no = item.value;
                    }
                    if (item.tag.equals("vcRouteName")) {
                        // コースの名称
                        if (tmpBusRouteInfo == null) {
                            ret = String.format(Locale.JAPAN, "バス経路情報モデルが存在しません。 %s=%s", item.tag, item.value);
                            EsnDebugLog.outputLog(ret, EsnDebugLog.DIV.ERR);
                            break;
                        }
                        if (item.value.length() == 0) {
                            ret = String.format(Locale.JAPAN, "BusRouteNameの値が正常でない。 %s=%s", item.tag, item.value);
                            EsnDebugLog.outputLog(ret, EsnDebugLog.DIV.ERR);
                            break;
                        }
                        tmpBusRouteInfo.bus_route_name = item.value;
                    }
                    if (tmpBusRouteInfo != null)
                    {
                        if (!tmpBusRouteInfo.bus_route_name.equals(""))
                        {
                            tmpBusRouteInfos.add(tmpBusRouteInfo);
                            tmpBusRouteInfo = null;
                        }
                    }
                }
                if (ret == null) {
                    // コース情報設定
                    _busRoute_infos.items = tmpBusRouteInfos;
                }
            }
        } catch (Exception e1) {
            EsnDebugLog.outputLog(String.format(Locale.JAPAN, "例外エラー１ %s", e1.toString()), EsnDebugLog.DIV.ERR);
            ret = getString(R.string.msgCommunicationError);
        } finally {
            try {
                if (is != null) is.close();
                if (os != null) os.close();
                if (con != null) con.disconnect();
            } catch (Exception e2) {
                EsnDebugLog.outputLog(String.format(Locale.JAPAN, "例外エラー２ %s", e2.toString()), EsnDebugLog.DIV.ERR);
                ret = getString(R.string.msgCommunicateCmpltError);
            }
        }
        return ret;
    }
    //endregion
    //endregion
    //region イベントメソッド
    /**
     * アクティビティ作成イベント
     *
     * @param savedInstanceState アクティビティが以前にシャットダウンされた後に再初期化されている場合、このバンドルにはonSaveInstanceState（Bundle）で最後に提供されたデータが含まれます。 注：それ以外の場合はnullです。
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_course_settings);
        _app.drive_state_model.init();
        // メンバ変数設定
        _spnCourseSettings = findViewById(R.id.spnCourseSelect);
        _btnSave = findViewById(R.id.btnSave);
        _btnUdpateCourse = findViewById(R.id.btnUpdateCourse);
        _prgUpdateCourse = findViewById(R.id.prgUpdateCourse);
        _org_model = new course_settings();
        _tmp_model = _org_model.clone();
        _busRoute_infos = new busRoute_infos();
        _system_settings = new system_settings();
        _communicating = false;

        // ボタンのリスナ登録
        findViewById(R.id.imbBack).setOnClickListener(this);
        _btnSave.setOnClickListener(this);
        _btnUdpateCourse.setOnClickListener(this);
        // 日付が変わったときにコースを更新するよう要求
        SimpleDateFormat fmtYYYYMMDD = new SimpleDateFormat("yyyyMMdd", Locale.JAPANESE);
        String nowDate = fmtYYYYMMDD.format(new Date());
        if ((_app.drive_state_model.getLastUpdateCourseDate() == null) || (_app.drive_state_model.getLastUpdateCourseDate().compareTo(nowDate) < 0)) {
            _org_model.initSelectedCourse();
            _tmp_model = _org_model.clone();
            _busRoute_infos.initBusRouteInfos();
            // コース更新要求ダイアログ
            DialogFragment newFragment = new NoticeDialogFragment(getString(R.string.titleCfm),
                    getString(R.string.msgResetCourse),
                    false,
                    false,
                    getString(R.string.ok),
                    "");
            newFragment.show(getSupportFragmentManager(), DIALOG_TAG_RESET_COURSE);
        }
        // コース選択スピナーを設定
        BusRouteAdapter adp2 = new BusRouteAdapter(this, (List<busRoute_info>)_busRoute_infos.items);
        _spnCourseSettings.setAdapter(adp2);
        _spnCourseSettings.setSelection(_tmp_model.selected_position);
        _spnCourseSettings.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener(){
            @Override
            public void onItemSelected(AdapterView<?> parent,  View view,  int position,  long id) {
                // 選択時の処理
                _tmp_model.selected_busRoute_info = (busRoute_info) parent.getSelectedItem();
                _tmp_model.selected_position = position;
                _tmp_model.savePreference();
                refreshButton();
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {
                // 項目が撰択されなかった場合の処理
            }
        }) ;
        // ボタンリフレッシュ
        refreshButton();
    }
    /**
     * クリックリスナー
     * @param view 対象のビュー
     */
    public void onClick(View view) {
        Intent intent;
        switch (view.getId()) {
            case R.id.imbBack: clickBtnBack(); break;   // 戻る
            case R.id.btnSave: clickBtnSave(); break; // 保存ボタン
            case R.id.btnUpdateCourse: clickBtnUpdateCourse(); break; // コース更新ボタン
        }
    }
    /**
     * ダイアログのOKボタン押下イベント
     * @param dialog ダイアログ
     */
    @Override
    public void onDialogPositiveClick(NoticeDialogFragment dialog) {
        String tag = dialog.getTag();
        if (tag == null) {
            EsnDebugLog.outputLog("ダイアログタグが設定されていない", EsnDebugLog.DIV.ERR);
            return;
        }
        EsnDebugLog.outputLog(String.format(Locale.JAPAN, "POSSTIVEボタン押下 %s", tag), EsnDebugLog.DIV.INFO);

        switch (tag) {
            case DIALOG_TAG_SAVE:
                // 保存確認ダイアログではいボタン押下
                clickPosCfmSave();
                break;
            case DIALOG_TAG_UPDATE_COURSE:
                // コース更新確認ダイアログではいボタン押下
                clickPosCfmUpdateCourse();
                break;
        }
    }
    /**
     * ダイアログのキャンセルボタン押下イベント
     * @param dialog ダイアログ
     */
    @Override
    public void onDialogNegativeClick(NoticeDialogFragment dialog) {
        String tag = dialog.getTag();
        if (tag == null) {
            EsnDebugLog.outputLog("ダイアログタグが設定されていない", EsnDebugLog.DIV.ERR);
            return;
        }
        EsnDebugLog.outputLog(String.format(Locale.JAPAN, "NEGATIVEボタン押下 %s", tag), EsnDebugLog.DIV.INFO);
    }
    /**
     * ダイアログのニュートラルボタン押下イベント
     * @param dialog ダイアログ
     */
    @Override
    public void onDialogNeutralClick(NoticeDialogFragment dialog) {
        String tag = dialog.getTag();
        if (tag == null) {
            EsnDebugLog.outputLog("ダイアログタグが設定されていない", EsnDebugLog.DIV.ERR);
            return;
        }
        EsnDebugLog.outputLog(String.format(Locale.JAPAN, "NEUTRALボタン押下 %s", tag), EsnDebugLog.DIV.INFO);
    }
    //endregion
}