package jp.co.ecosysnetwork.ccloca_app.activity;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.app.ActivityCompat;
import androidx.fragment.app.DialogFragment;

import android.Manifest;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.view.View;
import android.widget.TextView;

import java.util.Locale;

import jp.co.ecosysnetwork.ccloca_app.MyApplication;
import jp.co.ecosysnetwork.ccloca_app.NoticeDialogFragment;
import jp.co.ecosysnetwork.ccloca_app.R;
import jp.co.ecosysnetwork.universal.EsnDebugLog;
import jp.co.ecosysnetwork.universal.EsnUtility;

/**
 * スプラッシュアクティビティ
 */
public class SplashActivity
        extends AppCompatActivity
        implements View.OnClickListener,
                   NoticeDialogFragment.NoticeDialogListener {
    //region 定数
    /**
     * パーミッション要求用の識別コード
     */
    private static final int REQUEST_CODE = 200;
    /**
     * コース設定画面遷移時のパスワード入力ダイアログタグ
     */
    private static final String DIALOG_TAG_COURSE_SETTING_PASSWORD = "courseSettingPassword";
    /**
     * コース設定画面パスワード違いダイアログ
     */
    private static final String DIALOG_TAG_NEQ_COURSE_SETTING_PASSWORD = "neqCourseSettingPassword";
    /**
     * システム設定画面遷移時のパスワード入力ダイアログタグ
     */
    private static final String DIALOG_TAG_SYSTEM_SETTING_PASSWORD = "systemSettingPassword";
    /**
     * システム設定画面パスワード違いダイアログ
     */
    private static final String DIALOG_TAG_NEQ_SYSTEM_SETTING_PASSWORD = "neqSystemSettingPassword";
    //endregion
    //region メンバ変数
    /**
     * シングルトンのアプリケーション参照
     */
    private MyApplication _app = MyApplication.getInstance();
    /**
     * 自動スタートハンドラ
     */
    private Handler _autoStartHandler;
    //endregion
    //region 内部メソッド
    /**
     * 開始処理
     */
    private void startExec() {
        // メインアクティビティに遷移
        Intent intent = new Intent(getApplication(), MainActivity.class);
        startActivity(intent);

        // スプラッシュアクティビティは終了
        finish();
    }
    /**
     * 自動スタートハンドラ
     */
    private Runnable _autoStartRunnable = new Runnable() {
        @Override
        public void run() {
            EsnDebugLog.outputLog("自動スタート", EsnDebugLog.DIV.INFO);
            // 開始処理
            startExec();
        }
    };
    /**
     * 初期処理
     */
    private void initExec() {
        // ログ出力初期化
        EsnDebugLog.init(_app.getLogDirectory(), "ccloca_app_.log", 10, EsnDebugLog.DIV.DBG);

        // ログ出力
        Context ctx = _app.getApplicationContext();
        EsnDebugLog.outputLog("スプラッシュ起動 Ver." + EsnUtility.getVersionName(ctx), EsnDebugLog.DIV.INFO);

        // 10秒後にメインアクティビティに遷移する
        _autoStartHandler = new Handler(Looper.getMainLooper());
        _autoStartHandler.postDelayed(_autoStartRunnable, 10 * 1000);
    }
    /**
     * 開始ボタン押下処理
     */
    private void clickBtnStart() {
        EsnDebugLog.outputLog("開始ボタン押下", EsnDebugLog.DIV.INFO);
        // 開始処理
        startExec();
    }
    /**
     * コース設定パスワード入力ダイアログ－OKボタン押下処理
     * @param inp 入力テキスト
     */
    private void clickPosCourseSettingPassword(String inp) {
        if (_app.isOkCourseSettingPassword(inp)) {
            // コース設定アクティビティに遷移
            Intent intent = new Intent(getApplication(), CourseSettingsActivity.class);
            startActivity(intent);
        }
        else {
            EsnDebugLog.outputLog(String.format(Locale.JAPAN, "コース設定パスワード間違い %s", inp), EsnDebugLog.DIV.WARN);

            // パスワードエラーダイアログを表示
            DialogFragment newFragment = new NoticeDialogFragment(getString(R.string.titleCfm),
                    getString(R.string.msgNeqCourseSettingPassword),
                    false,
                    false,
                    getString(R.string.ok),
                    "");
            newFragment.show(getSupportFragmentManager(), DIALOG_TAG_NEQ_COURSE_SETTING_PASSWORD);
        }
    }
    /**
     * コース設定ボタン押下処理
     */
    private void clickBtnCourseSettings() {
        EsnDebugLog.outputLog("コース設定ボタン押下", EsnDebugLog.DIV.INFO);

        // パスワード入力ダイアログ表示
        DialogFragment newFragment = new NoticeDialogFragment(getString(R.string.courseSettings),
                getString(R.string.inpPassword),
                true,
                true,
                getString(R.string.ok),
                getString(R.string.cancel));
        newFragment.show(getSupportFragmentManager(), DIALOG_TAG_COURSE_SETTING_PASSWORD);
    }
    /**
     * システム設定パスワード入力ダイアログ－OKボタン押下処理
     * @param inp 入力テキスト
     */
    private void clickPosSystemSettingPassword(String inp) {
        if (_app.isOkSystemSettingPassword(inp)) {
            // システム設定アクティビティに遷移
            Intent intent = new Intent(getApplication(), SystemSettingsActivity.class);
            startActivity(intent);
        }
        else {
            EsnDebugLog.outputLog(String.format(Locale.JAPAN, "システム設定パスワード間違い %s", inp), EsnDebugLog.DIV.WARN);

            // パスワードエラーダイアログを表示
            DialogFragment newFragment = new NoticeDialogFragment(getString(R.string.titleCfm),
                    getString(R.string.msgNeqSystemSettingPassword),
                    false,
                    false,
                    getString(R.string.ok),
                    "");
            newFragment.show(getSupportFragmentManager(), DIALOG_TAG_NEQ_SYSTEM_SETTING_PASSWORD);
        }
    }
    /**
     * システム設定ボタン押下処理
     */
    private void clickBtnSystemSettings() {
        EsnDebugLog.outputLog("システム設定ボタン押下", EsnDebugLog.DIV.INFO);

        // パスワード入力ダイアログ表示
        DialogFragment newFragment = new NoticeDialogFragment(getString(R.string.systemSettings),
                getString(R.string.inpPassword),
                true,
                true,
                getString(R.string.ok),
                getString(R.string.cancel));
        newFragment.show(getSupportFragmentManager(), DIALOG_TAG_SYSTEM_SETTING_PASSWORD);
    }
    //endregion
    //region イベントメソッド
    /**
     * ダイアログのOKボタン押下イベント
     * @param dialog ダイアログ
     */
    @Override
    public void onDialogPositiveClick(NoticeDialogFragment dialog) {
        String tag = dialog.getTag();
        if (tag == null) {
            EsnDebugLog.outputLog("ダイアログタグが設定されていない", EsnDebugLog.DIV.ERR);
            return;
        }
        EsnDebugLog.outputLog(String.format(Locale.JAPAN, "POSSTIVEボタン押下 %s", tag), EsnDebugLog.DIV.INFO);

        switch (tag) {
            case DIALOG_TAG_COURSE_SETTING_PASSWORD:
                // コース設定確認ダイアログではいボタン押下
                clickPosCourseSettingPassword(dialog.get_edtInput().getText().toString());
                break;
            case DIALOG_TAG_SYSTEM_SETTING_PASSWORD:
                // システム設定確認ダイアログではいボタン押下
                clickPosSystemSettingPassword(dialog.get_edtInput().getText().toString());
                break;
        }
    }
    /**
     * ダイアログのキャンセルボタン押下イベント
     * @param dialog ダイアログ
     */
    @Override
    public void onDialogNegativeClick(NoticeDialogFragment dialog) {
        String tag = dialog.getTag();
        if (tag == null) {
            EsnDebugLog.outputLog("ダイアログタグが設定されていない", EsnDebugLog.DIV.ERR);
            return;
        }
        EsnDebugLog.outputLog(String.format(Locale.JAPAN, "NEGATIVEボタン押下 %s", tag), EsnDebugLog.DIV.INFO);
    }
    /**
     * ダイアログのニュートラルボタン押下イベント
     * @param dialog ダイアログ
     */
    @Override
    public void onDialogNeutralClick(NoticeDialogFragment dialog) {
        String tag = dialog.getTag();
        if (tag == null) {
            EsnDebugLog.outputLog("ダイアログタグが設定されていない", EsnDebugLog.DIV.ERR);
            return;
        }
        EsnDebugLog.outputLog(String.format(Locale.JAPAN, "NEUTRALボタン押下 %s", tag), EsnDebugLog.DIV.INFO);
    }
    /**
     * クリックリスナー
     * @param view 対象のビュー
     */
    public void onClick(View view) {
        // 自動スタートの解除
        _autoStartHandler.removeCallbacks(_autoStartRunnable);

        switch (view.getId()) {
            case R.id.btnStart: clickBtnStart(); break;   // 開始ボタン
            case R.id.btnCourseSettings: clickBtnCourseSettings(); break;   // コース設定ボタン
            case R.id.btnSystemSettings: clickBtnSystemSettings(); break;   // システム設定ボタン
        }
    }
    /**
     * アクティビティ作成イベント
     *
     * @param savedInstanceState アクティビティが以前にシャットダウンされた後に再初期化されている場合、このバンドルにはonSaveInstanceState（Bundle）で最後に提供されたデータが含まれます。 注：それ以外の場合はnullです。
     */
    @RequiresApi(api = Build.VERSION_CODES.R)
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_splash);

        // ボタンのリスナ登録
        findViewById(R.id.btnStart).setOnClickListener(this);
        findViewById(R.id.btnCourseSettings).setOnClickListener(this);
        findViewById(R.id.btnSystemSettings).setOnClickListener(this);

        // バージョン番号表示
        TextView txvVersion = findViewById(R.id.txvVersion);
        Context ctx = _app.getApplicationContext();
        String msg = getResources().getString(R.string.version) + EsnUtility.getVersionName(ctx);
        txvVersion.setText(msg);

        // パーミッションの付与
        if ((ActivityCompat.checkSelfPermission(ctx, Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED)
                || (ActivityCompat.checkSelfPermission(ctx, Manifest.permission.INTERNET) != PackageManager.PERMISSION_GRANTED)
                || (ActivityCompat.checkSelfPermission(ctx, Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED)
                || (ActivityCompat.checkSelfPermission(ctx, Manifest.permission.ACCESS_COARSE_LOCATION) != PackageManager.PERMISSION_GRANTED)
                || (ActivityCompat.checkSelfPermission(ctx, Manifest.permission.ACCESS_BACKGROUND_LOCATION) != PackageManager.PERMISSION_GRANTED)
        ) {
            // パーミッションがない場合、パーミッションを要求
            ActivityCompat.requestPermissions(this, new String[]{
//                    Manifest.permission.WRITE_EXTERNAL_STORAGE,
                    Manifest.permission.INTERNET,
                    Manifest.permission.ACCESS_FINE_LOCATION,
                    Manifest.permission.ACCESS_COARSE_LOCATION,
//                    Manifest.permission.ACCESS_BACKGROUND_LOCATION,
            }, REQUEST_CODE);
        } else {
            // 初期処理
            initExec();
        }
//            // 初期処理
//            initExec();
    }
    /**
     * パーミッション要求ダイアログで携帯端末が選択した際のイベント
     *
     * @param requestCode  要求コード
     * @param permissions  パーミッションリスト
     * @param grantResults 要求結果リスト
     */
    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        if (requestCode == REQUEST_CODE) {
//            if ((grantResults.length >= 5)
            if ((grantResults.length >= 3)
                    && (grantResults[0] == PackageManager.PERMISSION_GRANTED)
                    && (grantResults[1] == PackageManager.PERMISSION_GRANTED)
                    && (grantResults[2] == PackageManager.PERMISSION_GRANTED)
//                    && (grantResults[3] == PackageManager.PERMISSION_GRANTED)
//                    && (grantResults[4] == PackageManager.PERMISSION_GRANTED)
            ) {
                // パーミッションが得られた場合は処理を継続
                initExec();
            } else {
                // パーミッションが得られなかった場合は、プログラムを終了する
                finishAndRemoveTask();
            }
        }
    }
    //endregion
}