package jp.co.ecosysnetwork.ccloca_app.models;

import android.content.Context;
import android.content.SharedPreferences;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Objects;

import jp.co.ecosysnetwork.ccloca_app.MyApplication;
import jp.co.ecosysnetwork.universal.EsnDebugLog;

public class busRoute_infos {

    //region 外部公開メンバ変数
    /**
     * コース情報リスト
     */
    public List<busRoute_info> items;
    //endregion

    //region 生成と廃棄
    /*
     *  コンストラクタ
     */
    public busRoute_infos()
    {
        items = new ArrayList<>();
        Context ctx = MyApplication.getInstance().getApplicationContext();
        SharedPreferences pref = ctx.getSharedPreferences(MyApplication.PREFERENCE_NAME, Context.MODE_PRIVATE);

        busRoute_info info = new busRoute_info();
        info.bus_route_no = "0";
        info.bus_route_name = "コースを選択してください";
        items.add(info);

        // コース情報数の取得
        int infoNum = pref.getInt(MyApplication.PKEY_COURSE_INFO_NUM, 0);
        for (int i=0; i<infoNum; i++) {
            info = new busRoute_info();

            String key = String.format(Locale.JAPAN, MyApplication.PKEY_BUS_ROUTE_INFO_ID, i);
            info.bus_route_no = pref.getString(key, "0");

            key = String.format(Locale.JAPAN, MyApplication.PKEY_BUS_ROUTE_INFO_NAME, i);
            info.bus_route_name = pref.getString(key, "");

            if (!info.bus_route_no.equals("0"))
            {
                items.add(info);
            }
        }
    }
    /**
     * デバッグログ文字列の取得
     * @return デバッグログ文字列
     */
    public String toLogString() {
        StringBuilder ret = new StringBuilder();
        for (busRoute_info info : items) {
            ret.append(info.toLogString());
        }
        return ret.toString();
    }
    /**
     * 自身のクローンを返す
     * @return 自身のクローン
     */
    @Override
    public busRoute_infos clone() {
        busRoute_infos ret = null;
        try {
            ret = (busRoute_infos)super.clone();
            ret.items = new ArrayList<>();
            for (busRoute_info info : this.items) {
                ret.items.add(info.clone());
            }
        } catch (CloneNotSupportedException ex) {
            EsnDebugLog.outputLog("clone失敗 " + ex.toString(), EsnDebugLog.DIV.ERR);
        }
        return ret;
    }

    /**
     * 同値判定（Android Studioの自動生成機能を使用）
     * @param o 比較対象オブジェクト
     * @return true：同じ false：異なる
     */
    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        busRoute_infos that = (busRoute_infos) o;
        return items.equals(that.items);
    }

    /**
     * ハッシュコードを取得（Android Studioの自動生成機能を使用）
     * @return ハッシュコード
     */
    @Override
    public int hashCode() {
        return Objects.hash(items);
    }
    /**
     * プリファレンスへの保存
     * @return　true：成功　false：失敗
     */
    public boolean savePreference() {
        Context ctx = MyApplication.getInstance().getApplicationContext();
        SharedPreferences pref = ctx.getSharedPreferences(MyApplication.PREFERENCE_NAME, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = pref.edit();

        // コース情報数の登録
        editor.putInt(MyApplication.PKEY_COURSE_INFO_NUM, items.size());
        for (int i=0; i<items.size(); i++) {
            busRoute_info info = items.get(i);

            String key = String.format(Locale.JAPAN, MyApplication.PKEY_BUS_ROUTE_INFO_ID, i);
            editor.putString(key, info.bus_route_no);

            key = String.format(Locale.JAPAN, MyApplication.PKEY_BUS_ROUTE_INFO_NAME, i);
            editor.putString(key, info.bus_route_name);
        }
        return editor.commit();
    }
    /**
     * プリファレンスへの保存
     */
    public void initBusRouteInfos() {
        Context ctx = MyApplication.getInstance().getApplicationContext();
        SharedPreferences pref = ctx.getSharedPreferences(MyApplication.PREFERENCE_NAME, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = pref.edit();

        // コース情報数の登録
        editor.putInt(MyApplication.PKEY_COURSE_INFO_NUM, 0);
        for (int i=0; i<items.size(); i++) {
            String key = String.format(Locale.JAPAN, MyApplication.PKEY_BUS_ROUTE_INFO_ID, i);
            editor.putString(key, null);

            key = String.format(Locale.JAPAN, MyApplication.PKEY_BUS_ROUTE_INFO_NAME, i);
            editor.putString(key, null);
        }
        editor.commit();
    }
    /**
     * コース情報をIDから取得
     * @param bus_route_no コースキー
     * @return コース情報
     */
    public busRoute_info getCourseInfoFromInfoBusRouteNo(String bus_route_no) {
        busRoute_info ret = null;
        for (busRoute_info info : items) {
            if (info.bus_route_no.equals(bus_route_no) )  {
                ret = info;
                break;
            }
        }
        return ret;
    }
    //endregion
}
