package jp.co.ecosysnetwork.ccloca_app.thread;

import android.util.Xml;

import org.xmlpull.v1.XmlPullParser;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.math.BigDecimal;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Locale;

import jp.co.ecosysnetwork.ccloca_app.MyApplication;
import jp.co.ecosysnetwork.ccloca_app.models.location_send;
import jp.co.ecosysnetwork.universal.EsnDebugLog;
import jp.co.ecosysnetwork.universal.EsnQueue;
import jp.co.ecosysnetwork.universal.EsnXmlParser;

/**
 * サーバ通信スレッド
 */
public class CommThread  extends Thread
{
    //region 列挙体
    /**
     * 送信結果
     */
    private enum SEND_RESULT {
        /**
         * 成功
         */
        SUCCESS,
        /**
         * エラー
         */
        ERROR,
        /**
         * 携帯端末情報なし
         */
        NOT_EXIST_USER
    }
    //endregion
    //region 定数
    /**
     * 送信タイムアウト時間
     */
    private static final int SEND_TIMEOUT_MSEC = 5000;
    /**
     * 送信後の待機時間
     */
    private static final int WAIT_MSEC_SEND = 200;
    /**
     * メール再送信の最大回数
     */
    private static final int MAIL_SEND_RETRY_MAX = 20;
    /***
     * 送信実行間隔
     */
    private static final int EXECUTE_CYECLE_MSEC = 1000;
    //endregion
    //region メンバ変数
    /**
     * シングルトンのアプリケーション参照
     */
    private static final MyApplication _app = MyApplication.getInstance();
    /**
     * サーバURL
     */
    private String _url;
    /**
     * サーバ送信情報リスト
     */
    private location_send _location_send;
    //endregion
    //region 生成と廃棄
    /**
     * コンストラクタ
     * @param list サーバ送信情報リスト
     */
    public CommThread(String url, location_send list) {
        super();
        _url = url;
        _location_send = list;
    }
    //endregion
    //region 内部メソッド
    /**
     * サーバへの送信処理
     * @param location_send location_send
     * @return 送信結果
     */
    private SEND_RESULT sendExec(location_send location_send) {
        SEND_RESULT ret = SEND_RESULT.ERROR;
        // ログ出力処理開始タイム
        long startMsec = System.currentTimeMillis();
        // 例外エラー発生時にどこまで進んでいたかを判別するためのフラグ
        int step = 0;

        HttpURLConnection con = null;
        DataOutputStream os = null;
        InputStream is = null;
        InputStreamReader isr = null;
        BufferedReader br = null;
        try {
            URL accUrl = new URL(_url + MyApplication.APIURL_SEND_BUS_LOCATION);   step = 1;
            con = (HttpURLConnection)accUrl.openConnection();       step = 2;
            con.setConnectTimeout(SEND_TIMEOUT_MSEC);               step = 3;
            con.setReadTimeout(SEND_TIMEOUT_MSEC);                  step = 4;
            con.setRequestMethod("POST");                           step = 5;
            con.setDoOutput(true);                                  step = 6;
            con.setDoInput(true);                                   step = 7;
            con.setUseCaches(false);                                step = 8;
            // ヘッダ設定
            con.setRequestProperty("ANDROID-API-KEY", MyApplication.API_KEY); step = 9;
            con.setRequestProperty("Connection", "Keep-Alive");     step = 10;
            con.setRequestProperty("Charset", "UTF-8");             step = 11;
            String boundary = "-----------------------------" + System.currentTimeMillis(); step = 12;
            con.setRequestProperty("Content-Type", "multipart/form-data; boundary=" + boundary);
            con.connect();                                          step = 13;
            // 送信データの設定
            os = new DataOutputStream(con.getOutputStream());       step = 14;
            append_string(os, boundary, "intFacilityPhoneNo", location_send.phone_no); step = 15;
            append_string(os, boundary, "intBusRouteNo", location_send.bus_route_no);  step = 16;
            append_string(os, boundary, "decLatitude", BigDecimal.valueOf(location_send.latitude).toPlainString()); step = 17;
            append_string(os, boundary, "decLongitude", BigDecimal.valueOf(location_send.longitude).toPlainString()); step = 18;
            append_string(os, boundary, "dtDate", location_send.date.toString()); step = 19;
            append_final(os, boundary);                             step = 20;
            os = null;                                              step = 21;
            MyApplication.sleep(WAIT_MSEC_SEND);                    step = 22;
            // 受信データを取得
            is = con.getInputStream();                              step = 23;
            // 受信データのXMLパース
            XmlPullParser parser = Xml.newPullParser();             step = 24;
            parser.setInput(is, "UTF-8");               step = 25;

            // 受信データのパース
            EsnXmlParser esnParser = new EsnXmlParser(parser);      step = 26;
            String retParse = esnParser.tryParse();                 step = 27;
            ret = SEND_RESULT.SUCCESS;                              step = 28;
            String errMsg = "";                                     step = 29;
            if (retParse == null) {
                // パース成功
                for (EsnXmlParser.ParseItem item : esnParser.getParseItems()) {
                    if (item.tag.equals("errMsg")) {
                        EsnDebugLog.outputLog(String.format(Locale.JAPAN, "ret=%s phoneNo=%s busRouteNo = %s", item.value, location_send.phone_no, location_send.bus_route_no), EsnDebugLog.DIV.INFO);
                        errMsg = item.value;
                        ret = SEND_RESULT.ERROR;
                        break;
                    }
                }
            }                                                       step = 30;
            EsnDebugLog.outputLog(String.format(Locale.JAPAN, "ret=%s recv=%s time=%d url=%s", ret.toString(), errMsg, System.currentTimeMillis() - startMsec, _url), EsnDebugLog.DIV.INFO);
        }
        catch (Exception e1) {
            EsnDebugLog.outputLog(String.format(Locale.JAPAN, "例外エラー step=%d time=%d %s", step, System.currentTimeMillis() - startMsec, e1), EsnDebugLog.DIV.ERR);
            ret = SEND_RESULT.ERROR;
        }
        finally {
            try {if (br != null) br.close();} catch (Exception e2) {EsnDebugLog.outputLog(String.format(Locale.JAPAN, "br.close()エラー %s", e2.toString()), EsnDebugLog.DIV.ERR);}
            try {if (isr != null) isr.close();} catch (Exception e2) {EsnDebugLog.outputLog(String.format(Locale.JAPAN, "isr.close()エラー %s", e2.toString()), EsnDebugLog.DIV.ERR);}
            try {if (os != null) os.close();} catch (Exception e2) {EsnDebugLog.outputLog(String.format(Locale.JAPAN, "os.close()エラー %s", e2.toString()), EsnDebugLog.DIV.ERR);}
            try {if (is != null) is.close();} catch (Exception e2) {EsnDebugLog.outputLog(String.format(Locale.JAPAN, "is.close()エラー %s", e2.toString()), EsnDebugLog.DIV.ERR);}
            try {if (con != null) con.disconnect();} catch (Exception e2) {EsnDebugLog.outputLog(String.format(Locale.JAPAN, "con.disconnect()エラー %s", e2.toString()), EsnDebugLog.DIV.ERR);}
        }
        return ret;
    }
    /**
     * 送信文字列の追加
     * @param os データ出力ストリーム
     * @param boundary 境界
     * @param name 属性名
     * @param value 属性値
     * @throws IOException 入出力例外を返すことがある
     */
    protected void append_string(DataOutputStream os, String boundary, String name, String value) throws IOException {
        os.writeBytes("--" + boundary + "\r\n");
        os.writeBytes("Content-Disposition: form-data; name=" + "\"" + name + "\"" + "\r\n" + "\r\n");
        os.writeBytes(value + "\r\n");
    }
    /**
     * 送信データの終端設定
     * @param os データ入出力ストリーム
     * @param boundary 境界
     * @throws IOException 入出力例外を返すことがある
     */
    protected void append_final(DataOutputStream os, String boundary) throws IOException {
        os.writeBytes("--" + boundary + "--" + "\r\n");
        os.close();
    }
    //endregion
    //region 外部公開メソッド
    /**
     * サーバ通信処理
     */
    @Override
    public void run() {
        // 引き渡された送信データをキューに設定
        EsnQueue<location_send> location_send_queue = new EsnQueue<location_send>(100);
        if (_location_send != null) {
            location_send_queue.put(_location_send);
            EsnDebugLog.outputLog(String.format(Locale.JAPAN, "サーバ通信指示データ busRouteNo=%s", _location_send.bus_route_no), EsnDebugLog.DIV.INFO);
        }
        else {
            EsnDebugLog.outputLog("引き渡された送信データがnull", EsnDebugLog.DIV.ERR);
        }

        while (true) {
            // メッセージキューを取得
            location_send ps = location_send_queue.get();
            if (ps == null) {
                break;
            }

            // 前回時間
            _app.drive_state_model.setLastCommActiveMsec(System.currentTimeMillis());

            // サーバへの送信処理
            SEND_RESULT sendRet = sendExec(_location_send);

            if (sendRet == SEND_RESULT.ERROR) {
                if (ps.mail_send_cnt < MAIL_SEND_RETRY_MAX) {
                    ps.mail_send_cnt++;
                    if (location_send_queue.put(ps)) {
                        EsnDebugLog.outputLog(String.format(Locale.JAPAN, "メール再送信 bus_route_no=%s cnt=%d", ps.bus_route_no, ps.mail_send_cnt), EsnDebugLog.DIV.WARN);
                    }
                    else {
                        EsnDebugLog.outputLog(String.format(Locale.JAPAN, "キュー一杯のためメール再送信できず bus_route_no=%s cnt=%d", ps.bus_route_no,ps.mail_send_cnt), EsnDebugLog.DIV.ERR);
                    }
                }
                else {
                    EsnDebugLog.outputLog(String.format(Locale.JAPAN, "メール再送信回数オーバー bus_route_no=%s cnt=%d", ps.bus_route_no,ps.mail_send_cnt), EsnDebugLog.DIV.ERR);
                }
            }
            MyApplication.sleep(EXECUTE_CYECLE_MSEC);
        }
        EsnDebugLog.outputLog("サーバ通信終了", EsnDebugLog.DIV.INFO);
    }
    //endregion
}
