package jp.co.ecosysnetwork.universal;

import java.util.LinkedList;

/**
 * キュークラス
 * @param <E> レコードのクラス
 */
public class EsnQueue<E> {
    //region メンバ変数
    /**
     * キューリスト
     */
    private LinkedList<E> _queues;
    /**
     * 最大キューサイズ
     */
    private int _maxQueueSize;
    /**
     * 排他用オブジェクト
     */
    private final Object _lockObj = new Object();
    //endregion
    //region 生成と廃棄
    /**
     * コンストラクタ
     * @param maxQueueSize 最大キューサイズ
     */
    public EsnQueue(int maxQueueSize) {
        _queues = new LinkedList<E>();
        _maxQueueSize = maxQueueSize;
    }
    //endregion
    //region 外部公開メソッド
    /**
     * キューに追加
     * @param obj　追加レコード
     * @return ture：成功　false：最大数オーバー
     */
    public boolean put(E obj) {
        synchronized (_lockObj) {
            if (_queues.size() < _maxQueueSize) {
                _queues.addFirst(obj);
                return true;
            } else {
                return false;
            }
        }
    }
    /**
     * キューから抜き出し
     * @return レコード。レコードがない場合、null。
     */
    public E get() {
        synchronized (_lockObj) {
            if (_queues.size() > 0) {
                return _queues.removeLast();
            } else {
                return null;
            }
        }
    }
    //endregion
}
