package jp.co.ecosysnetwork.universal;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.text.InputFilter;

import java.io.File;
import java.io.FileInputStream;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * ECOSYSNETWORK K.K.  共通ユーティリティクラス
 */
public class EsnUtility {
    //region 定数
    /**
     * 大文字アルファベット
      */
    private static final String[] ALPHABETS = new String[]{
            "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"
    };
    /**
     * ファイル名として使用できない文字
     */
    public static final char[] PROHIBITED_SYMBOLS = {
            '\\', '/', ':', '*', '?', '\'', '\"', '<', '>'
    };
    //endregion
    //region 外部公開メソッド
    /**
     * 指定された前階層の実行中クラス名を取得します。
     *
     * @param preIdx 指定された前階層
     * @return クラス名
     */
    public static String getClassName(int preIdx) {
        return Thread.currentThread().getStackTrace()[preIdx].getClassName();
    }
    /**
     * 実行中のクラス名を取得します。
     *
     * @return クラス名
     */
    public static String getClassName() {
        return getClassName(3);
    }
    /**
     * 指定された前階層の実行中メソッド名を取得します。
     *
     * @param preIdx 指定された前階層
     * @return メソッド名
     */
    public static String getMethodName(int preIdx) {
        return Thread.currentThread().getStackTrace()[preIdx].getMethodName();
    }
    /**
     * 実行中のメソッド名を取得します。
     *
     * @return メソッド名
     */
    public static String getMethodName() {
        return getMethodName(3);
    }
    /**
     * ファイル名から拡張子を返します。
     *
     * @param fileName ファイル名
     * @return ファイルの拡張子
     */
    public static String getFileSuffix(String fileName) {
        if (fileName == null) {
            return null;
        }
        int point = fileName.lastIndexOf(".");
        if (point != -1) {
            return fileName.substring(point + 1);
        }
        return fileName;
    }
    /**
     * ファイル名から拡張子を取り除いた名前を返します。
     *
     * @param fileName ファイル名
     * @return ファイル名
     */
    public static String getFilePreffix(String fileName) {
        if (fileName == null) {
            return null;
        }
        int point = fileName.lastIndexOf(".");
        if (point != -1) {
            return fileName.substring(0, point);
        }
        return fileName;
    }
    /**
     * バージョンコードを取得する
     *
     * @param context コンテキスト
     * @return バージョンコード
     */
    public static int getVersionCode(Context context) {
        PackageManager pm = context.getPackageManager();
        int versionCode = 0;
        try {
            PackageInfo packageInfo = pm.getPackageInfo(context.getPackageName(), 0);
            versionCode = packageInfo.versionCode;
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
        return versionCode;
    }
    /**
     * バージョン名を取得する
     *
     * @param context コンテキスト
     * @return バージョン名
     */
    public static String getVersionName(Context context) {
        PackageManager pm = context.getPackageManager();
        String versionName = "";
        try {
            PackageInfo packageInfo = pm.getPackageInfo(context.getPackageName(), 0);
            versionName = packageInfo.versionName;
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
        return versionName;
    }
    /**
     * 配列から値を検索し、発見したインデックスを返す
     * @param array 配列
     * @param find 検索値
     * @return 発見したインデックス。-1：見つからない
     */
    public static int indexOf(int[] array, int find) {
        int ret = -1;
        for (int i=0; i<array.length; i++) {
            if (array[i] == find) {
                ret = i;
                break;
            }
        }
        return ret;
    }
    /**
     * 指定パスのBMPファイルを読込BITMAP形式で返す
     * @param filePath ファイルパス
     * @return BITMAPデータ
     */
    public static Bitmap readBitmapFile(String filePath) {
        Bitmap ret = null;
        try {
            File srcFile = new File(filePath);
            if (srcFile.exists()) {
                FileInputStream fis = new FileInputStream(srcFile);
                ret = BitmapFactory.decodeStream(fis);
            }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
        return ret;
    }
    /**
     * 文字列が数値のみであるか判定
     * @param target 文字列
     * @return true：数値である false：数値でない
     */
    public static boolean isNumeric(String target) {
        boolean ret = false;
        for (int i = 0; i < target.length(); i++) {
            ret = Character.isDigit(target.charAt(i));
            if (!ret) {
                break;
            }
        }
        return ret;
    }
    /**
     * メールアドレスとして正しい文字列か判定
     * @param address メールアドレス
     * @return true：正しい false：正しくない
     */
    public static boolean isOkMailAddress(String address) {
        boolean ret;
        String aText = "[\\w!#%&'/=~`\\*\\+\\?\\{\\}\\^\\$\\-\\|]";
        String dotAtom = aText + "+" + "(\\." + aText + "+)*";
        String regularExpression = "^" + dotAtom + "@" + dotAtom + "$";
        Pattern pattern = Pattern.compile(regularExpression);
        Matcher matcher = pattern.matcher(address);
        return matcher.find();
    }
    /**
     * ファイル名として正しい文字列か判定
     * @param fileName ファイル名
     * @return true：正しい false：正しくない
     */
    public static boolean isOkFileName(String fileName) {
        if (fileName.length() == 0) {
            return false;
        }
        boolean ret = true;
        for (int i = 0; i < fileName.length(); i++) {
            for (char c : PROHIBITED_SYMBOLS) {
                if (fileName.charAt(i) == c) {
                    ret = false;
                    break;
                }
            }
            if (!ret) {
                break;
            }
        }
        return ret;
    }
    /**
     * リスト要素の入替
     * @param list リスト
     * @param index1 入替えるインデックス1
     * @param index2 入替えるインデックス2
     * @param <T> リストの型
     */
    public static<T> void swap(List<T> list, int index1, int index2) {
        T tmp=list.get(index1);
        list.set(index1,list.get(index2));
        list.set(index2, tmp);
    }
    /**
     * 緯度、経度から２点間の距離を算出
     * @param lat1 緯度1
     * @param lon1 経度1
     * @param lat2 緯度2
     * @param lon2 経度2
     * @return ２点間の距離（単位：メートル）
     */
    public static double CalcDistanceFromLocation(double lat1, double lon1, double lat2, double lon2) {
        double ret = 0;
        if ((lat1 > 0) && (lon1 > 0) && (lat2 > 0) && (lon2 > 0)) {
            final double M_PI = 3.14159265358979;
            // 始点緯度をラジアンに変換
            double dSirad = lat1 * M_PI / 180;
            // 始点経度をラジアンに変換
            double dSkrad = lon1 * M_PI / 180;

            // 終点緯度をラジアンに変換
            double dSyirad = lat2 * M_PI / 180;
            // 終点経度をラジアンに変換
            double dSykrad = lon2 * M_PI / 180;

            //２点間の平均緯度を計算
            double dAveirad = (dSirad + dSyirad) / 2;

            //２点間の緯度差を計算
            double dDeffirad = dSirad - dSyirad;

            //２点間の経度差を計算
            double dDeffkrad = dSkrad - dSykrad;

            //GRS80楕円体 (2000年系)
            //子午線曲率半径を計算
            double dTemp = 1 - 0.006694 * (Math.sin(dAveirad) * Math.sin(dAveirad));
            double dDmrad = 6335439 / Math.sqrt(dTemp * dTemp * dTemp);
            //卯酉線曲率半径を取得
            double dDvrad = 6378137 / Math.sqrt(dTemp);

            //ヒュベニの距離計算式
            double dT1 = dDmrad * dDeffirad;
            double dT2 = dDvrad * Math.cos(dAveirad) * dDeffkrad;
            ret = Math.sqrt(dT1 * dT1 + dT2 * dT2);
        }
        return ret;
    }
    /**
     * 緯度、経度から２点間の角度を算出
     * @param lat1 緯度1
     * @param lon1 経度1
     * @param lat2 緯度2
     * @param lon2 経度2
     * @return ２点間の角度（0~360度）
     */
    public static double CalcAngleFromLocation(double lat1, double lon1, double lat2, double lon2) {
        double ret = 0;
        if ((lat1 > 0) && (lon1 > 0) && (lat2 > 0) && (lon2 > 0)) {
            double radLat1 = Math.toRadians(lat2);
            double radLon1 = Math.toRadians(lon2);
            double radLat2 = Math.toRadians(lat1);
            double radLon2 = Math.toRadians(lon1);
            double Y = Math.sin(radLon2 - radLon1) * Math.cos(radLat2);
            double X = Math.cos(radLat1) * Math.sin(radLat2) - Math.sin(radLat1) * Math.cos(radLat2) * Math.cos(radLon2 - radLon1);
            double deg = Math.toDegrees(Math.atan2(Y, X));
            double angle = (deg + 360) % 360;

            ret = (Math.abs(angle) + ((double)1 / 7200));
        }
        return ret;
    }
    //endregion
}
