package jp.co.ecosysnetwork.universal;

import android.content.SharedPreferences;

import org.xmlpull.v1.XmlPullParser;

import java.util.ArrayList;
import java.util.List;

/**
 * XMLパーサ
 */
public class EsnXmlParser {
    //region クラス
    /**
     * 解析項目
     */
    public class ParseItem {
        /**
         * タグ
         */
        public String tag;
        /**
         * 値
         */
        public String value;
        //region 生成と廃棄
        /**
         * コンストラクタ
         */
        public ParseItem() {
            this.tag = "";
            this.value = "";
        }
        /**
         * コンストラクタ
         * @param tag タグ
         * @param value 値
         */
        public ParseItem(String tag, String value) {
            this.tag = tag;
            this.value = value;
        }
        //endregion
    }
    //endregion
    //region メンバ変数
    /**
     * XMLパーサ
     */
    private XmlPullParser _parser;
    /**
     * 解析項目リスト
     */
    private List<ParseItem> _items;
    //endregion
    //region 生成と廃棄
    /**
     * コンストラクタ
     * @param parser XMLパーサ
     */
    public EsnXmlParser(XmlPullParser parser) {
        _parser = parser;
        _items = new ArrayList<>();
    }
    //endregion
    //region 外部公開メソッド
    /**
     * 解析実行
     * @return エラーメッセージ
     */
    public String tryParse() {
        String ret = null;
        _items = new ArrayList<>();
        try {
            int et = _parser.getEventType();
            boolean cmplt = false;
            for (int i=0; i<10000; i++) {     // 最大回数に保険をかける
                // 解析終了すると抜ける
                if (et == XmlPullParser.END_DOCUMENT) {
                    cmplt = true;
                    break;
                }
                if (et == XmlPullParser.START_TAG) {
                    String tag = _parser.getName();
                    String value = "";
                    boolean cmplt2 = false;
                    for (int j=0; j<100; j++) { // 最大回数に保険をかける
                        if (et == XmlPullParser.TEXT) {
                            value = _parser.getText();
                            cmplt2 = true;
                            break;
                        }
                        et = _parser.next();
                    }
                    if (cmplt2) {
                        _items.add(new ParseItem(tag, value));
                        et = _parser.next();
                    }
                    else {
                        ret = "タグに該当する値が存在しない タグ=" + tag;
                        return ret;
                    }
                } else if (et == XmlPullParser.END_TAG) {
                    et = _parser.next();
                } else if (et == XmlPullParser.START_DOCUMENT) {
                    et = _parser.next();
                } else {
                    et = _parser.next();
                }
            }
            if (!cmplt) {
                ret = "終了タグが存在しない";
            }
        }
        catch (Exception e) {
            ret = "例外エラー " + e.toString();
        }
        return ret;
    }
    /**
     * 解析項目リストを取得
     * @return 解析項目リスト
     */
    public List<ParseItem> getParseItems() {
        return _items;
    }
    //endregion
}
